import {
  useFreshRef
} from "./chunk-KTZM67PO.js";

// src/unstable-use-frame-state.ts
import { useMemo, useReducer, useRef } from "react";
function computeDurationInSeconds(start, end) {
  return Number(((end.getTime() - start.getTime()) / 1e3).toFixed(2));
}
function createFramesStackReducer(resolveSpecificationRef) {
  return function framesStackReducer(state, action) {
    switch (action.action) {
      case "LOAD":
        return {
          ...state,
          stack: [action.item, ...state.stack]
        };
      case "DONE_REDIRECT": {
        const index = state.stack.findIndex(
          (item) => item.timestamp === action.pendingItem.timestamp
        );
        if (index === -1) {
          return state;
        }
        state.stack[index] = {
          ...action.pendingItem,
          ...action.item,
          status: "doneRedirect"
        };
        return {
          ...state,
          stack: state.stack.slice()
        };
      }
      case "DONE_WITH_ERROR_MESSAGE":
      case "DONE_WITH_MESSAGE": {
        const index = state.stack.findIndex(
          (item) => item.timestamp === action.pendingItem.timestamp
        );
        if (index === -1) {
          return state;
        }
        state.stack[index] = {
          ...action.pendingItem,
          ...action.item
        };
        return {
          ...state,
          stack: state.stack.slice()
        };
      }
      case "DONE": {
        const index = state.stack.findIndex(
          (item) => item.timestamp === action.pendingItem.timestamp
        );
        if (index === -1) {
          return state;
        }
        let signerState;
        let specification;
        let frameContext;
        let homeframeUrl;
        let parseResult = action.parseResult;
        if (state.type === "not-initialized") {
          const resolvedSpecification = resolveSpecificationRef.current({
            parseResult: action.parseResult
          });
          ({ signerState, specification, frameContext } = resolvedSpecification);
          homeframeUrl = action.pendingItem.url;
        } else {
          ({
            signerState,
            specification,
            frameContext,
            homeframeUrl,
            parseResult
          } = state);
        }
        state.stack[index] = {
          ...action.pendingItem,
          status: "done",
          speed: computeDurationInSeconds(
            action.pendingItem.timestamp,
            action.endTime
          ),
          frameResult: action.parseResult[specification],
          response: action.response,
          responseStatus: action.response.status,
          responseBody: action.parseResult
        };
        return {
          ...state,
          parseResult,
          signerState,
          frameContext,
          homeframeUrl,
          specification,
          type: "initialized",
          stack: state.stack.slice()
        };
      }
      case "REQUEST_ERROR": {
        const index = state.stack.findIndex(
          (item) => item.timestamp === action.pendingItem.timestamp
        );
        if (index === -1) {
          return state;
        }
        state.stack[index] = action.item;
        return {
          ...state,
          stack: state.stack.slice()
        };
      }
      case "RESET": {
        if (state.type === "not-initialized") {
          return state;
        }
        const { frameContext, signerState, specification } = resolveSpecificationRef.current({ parseResult: state.parseResult });
        return {
          ...state,
          stack: !!state.stack[0] && state.stack.length > 0 ? [state.stack[0]] : [],
          type: "initialized",
          frameContext,
          signerState,
          specification
        };
      }
      case "RESET_INITIAL_FRAME": {
        const { frameContext, signerState, specification } = resolveSpecificationRef.current({ parseResult: action.parseResult });
        const frameResult = action.parseResult[specification];
        return {
          type: "initialized",
          signerState,
          frameContext,
          specification,
          homeframeUrl: action.homeframeUrl,
          parseResult: action.parseResult,
          stack: [
            {
              request: {
                method: "GET",
                url: action.homeframeUrl
              },
              url: action.homeframeUrl,
              requestDetails: {},
              response: new Response(JSON.stringify(frameResult), {
                status: 200,
                headers: { "Content-Type": "application/json" }
              }),
              responseStatus: 200,
              timestamp: /* @__PURE__ */ new Date(),
              speed: 0,
              frameResult,
              status: "done",
              responseBody: frameResult
            }
          ]
        };
      }
      case "CLEAR":
        return {
          type: "not-initialized",
          stack: []
        };
      default:
        return state;
    }
  };
}
function useFrameState({
  initialParseResult,
  initialFrameUrl,
  resolveSpecification
}) {
  const resolveSpecificationRef = useFreshRef(resolveSpecification);
  const reducerRef = useRef(createFramesStackReducer(resolveSpecificationRef));
  const [state, dispatch] = useReducer(
    reducerRef.current,
    [initialParseResult, initialFrameUrl],
    ([parseResult, frameUrl]) => {
      if (parseResult && frameUrl) {
        const { frameContext, signerState, specification } = resolveSpecification({
          parseResult
        });
        const frameResult = parseResult[specification];
        return {
          type: "initialized",
          frameContext,
          signerState,
          specification,
          homeframeUrl: frameUrl,
          parseResult,
          stack: [
            {
              response: new Response(JSON.stringify(frameResult), {
                status: 200,
                headers: { "Content-Type": "application/json" }
              }),
              responseStatus: 200,
              responseBody: frameResult,
              timestamp: /* @__PURE__ */ new Date(),
              requestDetails: {},
              request: {
                method: "GET",
                url: frameUrl
              },
              speed: 0,
              frameResult,
              status: "done",
              url: frameUrl
            }
          ]
        };
      }
      return {
        type: "not-initialized",
        stack: frameUrl ? [
          // prevent flash of empty content by adding pending item because initial frame is being loaded
          {
            method: "GET",
            request: {
              method: "GET",
              url: frameUrl
            },
            url: frameUrl,
            requestDetails: {},
            timestamp: /* @__PURE__ */ new Date(),
            status: "pending"
          }
        ] : []
      };
    }
  );
  const api = useMemo(() => {
    return {
      dispatch,
      clear() {
        dispatch({
          action: "CLEAR"
        });
      },
      createGetPendingItem(arg) {
        const item = {
          method: "GET",
          request: arg.request,
          requestDetails: {},
          url: arg.request.url,
          timestamp: /* @__PURE__ */ new Date(),
          status: "pending"
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      createPostPendingItem(arg) {
        const item = {
          method: "POST",
          request: arg.request,
          requestDetails: {
            body: arg.action.body,
            searchParams: arg.action.searchParams
          },
          url: arg.action.searchParams.get("postUrl") ?? "missing postUrl",
          timestamp: arg.startTime ?? /* @__PURE__ */ new Date(),
          status: "pending"
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      createCastOrComposerActionPendingItem(arg) {
        return {
          method: "POST",
          requestDetails: {
            body: arg.action.body,
            searchParams: arg.action.searchParams
          },
          request: arg.request,
          status: "pending",
          timestamp: /* @__PURE__ */ new Date(),
          url: arg.action.searchParams.get("postUrl") ?? "missing postUrl"
        };
      },
      markCastFrameAsDone() {
      },
      markCastMessageAsDone(arg) {
        dispatch({
          action: "LOAD",
          item: arg.pendingItem
        });
        dispatch({
          action: "DONE_WITH_MESSAGE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "message",
            message: arg.responseData.message,
            response: arg.response.clone(),
            responseBody: arg.responseData,
            responseStatus: arg.response.status,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            type: "info"
          }
        });
      },
      markComposerFormActionAsDone() {
      },
      markAsDone(arg) {
        dispatch({
          action: "DONE",
          pendingItem: arg.pendingItem,
          parseResult: arg.parseResult,
          response: arg.response.clone(),
          endTime: arg.endTime
        });
      },
      markAsDoneWithErrorMessage(arg) {
        dispatch({
          action: "DONE_WITH_ERROR_MESSAGE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            responseStatus: arg.response.status,
            response: arg.response.clone(),
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            status: "message",
            type: "error",
            message: arg.responseData.message,
            responseBody: arg.responseData
          }
        });
      },
      markAsDoneWithRedirect(arg) {
        dispatch({
          action: "DONE_REDIRECT",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            location: arg.location,
            response: arg.response.clone(),
            responseBody: arg.responseBody,
            responseStatus: arg.response.status,
            status: "doneRedirect",
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            )
          }
        });
      },
      markAsFailed(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            request: arg.pendingItem.request,
            requestDetails: arg.pendingItem.requestDetails,
            timestamp: arg.pendingItem.timestamp,
            url: arg.pendingItem.url,
            response: arg.response?.clone() ?? null,
            responseStatus: arg.responseStatus,
            requestError: arg.requestError,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            status: "requestError",
            responseBody: arg.responseBody
          }
        });
      },
      markAsFailedWithRequestError(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "requestError",
            requestError: arg.error,
            response: arg.response.clone(),
            responseStatus: arg.response.status,
            responseBody: arg.responseBody,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            )
          }
        });
      },
      reset(arg) {
        if (!arg) {
          dispatch({ action: "RESET" });
        } else {
          dispatch({
            action: "RESET_INITIAL_FRAME",
            homeframeUrl: arg.homeframeUrl,
            parseResult: arg.parseResult
          });
        }
      }
    };
  }, [dispatch]);
  return [state, api];
}

export {
  useFrameState
};
