import {
  WebStorage
} from "./chunk-YWADTQF4.js";

// src/hooks/use-storage.ts
import { useCallback, useEffect, useRef, useState } from "react";
var defaultStorage = new WebStorage();
function useStorage({
  key,
  initialValue,
  preprocessValue,
  storage = defaultStorage
}) {
  const storageRef = useRef(storage);
  const preprocessValueRef = useRef(preprocessValue);
  preprocessValueRef.current = preprocessValue;
  const initialValueRef = useRef(initialValue);
  const [value, setValue] = useState(
    initialValueRef.current
  );
  useEffect(() => {
    function isValueNotUndefined(newValue) {
      return newValue !== void 0;
    }
    const listener = (newValue) => {
      if (!isValueNotUndefined(newValue)) {
        setValue(initialValueRef.current);
        return;
      }
      setValue(
        preprocessValueRef.current ? preprocessValueRef.current(newValue) : newValue
      );
    };
    const unsubscribe = storageRef.current.watch(key, listener);
    return () => {
      unsubscribe();
    };
  }, [key]);
  const setState = useCallback(
    async (setter) => {
      await storageRef.current.set(key, (currentState) => {
        if (typeof setter !== "function") {
          return setter;
        }
        return setter(
          currentState === void 0 ? initialValueRef.current : currentState
        );
      });
    },
    [key]
  );
  return [value, setState];
}

export {
  useStorage
};
