// src/identity/farcaster/identity-poller.ts
var POLL_INTERVAL = 2e3;
var IdentityPoller = class {
  controller;
  polling = false;
  constructor() {
    this.controller = new AbortController();
  }
  isPolling = () => {
    return this.polling;
  };
  start = async (token) => {
    this.controller = new AbortController();
    this.polling = true;
    while (true) {
      if (this.controller.signal.aborted) {
        this.polling = false;
        return false;
      }
      try {
        const fcSignerRequestResponse = await fetch(
          `https://api.warpcast.com/v2/signed-key-request?token=${token}`,
          {
            method: "GET",
            headers: {
              "Content-Type": "application/json"
            }
          }
        );
        if (fcSignerRequestResponse.ok) {
          const responseBody = await fcSignerRequestResponse.json();
          if (responseBody.result.signedKeyRequest.state === "completed") {
            return responseBody.result.signedKeyRequest;
          }
        } else {
          throw new Error(
            `Warpcast API returned an error response with status code: ${fcSignerRequestResponse.status}`
          );
        }
      } catch (error) {
        console.error(
          "Error while polling for the signer approval status",
          error
        );
      }
      await new Promise((r) => {
        setTimeout(r, POLL_INTERVAL);
      });
    }
  };
  stop = () => {
    if (this.controller.signal.aborted) {
      return;
    }
    this.controller.abort();
  };
};

export {
  IdentityPoller
};
