import {
  isSpecificationValid
} from "./chunk-XY5MZVO3.js";

// src/next/GET.tsx
import { parseFramesWithReports } from "frames.js/parseFramesWithReports";
import { getFrame } from "frames.js";
async function GET(request) {
  try {
    const searchParams = "nextUrl" in request ? request.nextUrl.searchParams : new URL(request.url).searchParams;
    const url = searchParams.get("url");
    const specification = searchParams.get("specification") ?? "farcaster";
    const multiSpecificationEnabled = searchParams.get("multispecification") === "true";
    if (!url) {
      return Response.json({ message: "Invalid URL" }, {
        status: 400
      });
    }
    const urlRes = await fetch(url);
    const html = await urlRes.text();
    if (multiSpecificationEnabled) {
      const result2 = parseFramesWithReports({
        html,
        fallbackPostUrl: url
      });
      return Response.json(result2);
    }
    if (!isSpecificationValid(specification)) {
      return Response.json(
        { message: "Invalid specification" },
        {
          status: 400
        }
      );
    }
    const result = getFrame({
      htmlString: html,
      url,
      specification
    });
    return Response.json(result);
  } catch (err) {
    console.error(err);
    return Response.json({ message: String(err) }, {
      status: 500
    });
  }
}

export {
  GET
};
