import { FarcasterSignerState } from '../../farcaster/signers.cjs';
import { Storage } from '../types.cjs';
import { useVisibilityDetection } from '../../hooks/use-visibility-detection.cjs';
import { IdentityPoller } from './identity-poller.cjs';
import { FarcasterSigner, FarcasterCreateSignerResult, FarcasterSignerImpersonating, FarcasterSignerApproved, FarcasterSignerPendingApproval } from './types.cjs';
import '../../types-BvcGM4gO.cjs';
import 'frames.js';
import 'react';
import 'frames.js/frame-parsers';
import 'frames.js/types';
import '../../farcaster/types.cjs';

type UseFarcasterIdentityOptions = {
    /**
     * Called when it is required to create a new signer in order to proceed
     */
    onMissingIdentity: () => void;
    /**
     * Allows you to disable the polling of the signer approval status
     * when user starts signin in
     *
     * @defaultValue true
     */
    enableIdentityPolling?: boolean;
    /**
     * URL to signer endpoint
     *
     * @defaultValue '/signer'
     */
    signerUrl?: string;
    /**
     * @defaultValue WebStorage
     */
    storage?: Storage;
    /**
     * @defaultValue 'farcasterIdentity'
     */
    storageKey?: string;
    generateUserId?: () => string | number;
    /**
     * Used to detect if the current context is visible, this affects the polling of the signer approval status.
     */
    visibilityChangeDetectionHook?: typeof useVisibilityDetection;
    onImpersonate?: (identity: FarcasterSignerImpersonating) => void;
    onLogIn?: (identity: FarcasterSignerApproved) => void;
    onLogInStart?: (identity: FarcasterSignerPendingApproval) => void;
    onLogOut?: (identity: FarcasterSigner) => void;
};
type FarcasterSignerInstance = FarcasterSignerState<FarcasterSigner | null> & {
    createSigner: () => Promise<FarcasterCreateSignerResult>;
    impersonateUser: (fid: number) => Promise<void>;
    identityPoller: IdentityPoller;
};
declare function useFarcasterIdentity({ onMissingIdentity, enableIdentityPolling, signerUrl, storage, storageKey, visibilityChangeDetectionHook, onImpersonate, onLogIn, onLogInStart, onLogOut, generateUserId, }: UseFarcasterIdentityOptions): FarcasterSignerInstance;

export { type FarcasterSignerInstance, useFarcasterIdentity };
