"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/identity/xmtp/use-xmtp-identity.tsx
var use_xmtp_identity_exports = {};
__export(use_xmtp_identity_exports, {
  useXmtpIdentity: () => useXmtpIdentity
});
module.exports = __toCommonJS(use_xmtp_identity_exports);
var import_rainbowkit = require("@rainbow-me/rainbowkit");
var import_frames_client = require("@xmtp/frames-client");
var import_xmtp_js = require("@xmtp/xmtp-js");
var import_react = require("react");
var import_viem = require("viem");
var import_wagmi = require("wagmi");
var import_actions = require("wagmi/actions");

// src/identity/storage/web-storage.ts
var localStorageUnavailableMessage = "WebStorage: localStorage is not available, consider implementing your own Storage class";
var WebStorage = class {
  /**
   * Keeps latest values in memory so we don't need to read them every time
   * we set or read the value.
   */
  values;
  watchers = {};
  constructor() {
    this.values = /* @__PURE__ */ new Map();
    this.watchers = {};
  }
  get = async (key) => {
    if (this.values.has(key)) {
      return Promise.resolve(this.values.get(key));
    }
    if (typeof localStorage !== "undefined") {
      const value = localStorage.getItem(key);
      if (!value) {
        return Promise.resolve(void 0);
      }
      const parsedValue = JSON.parse(value);
      this.values.set(key, parsedValue);
      return Promise.resolve(parsedValue);
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve(void 0);
  };
  set = async (key, setter) => {
    const currentValue = await this.get(key);
    const newValue = setter(currentValue);
    this.values.set(key, newValue);
    this.notifyChange(key, newValue);
    if (typeof localStorage !== "undefined") {
      localStorage.setItem(key, JSON.stringify(newValue));
      return;
    }
    console.warn(localStorageUnavailableMessage);
  };
  delete = (key) => {
    this.values.delete(key);
    this.notifyChange(key, void 0);
    if (typeof localStorage !== "undefined") {
      localStorage.removeItem(key);
      return Promise.resolve();
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve();
  };
  watch = (key, listener) => {
    this.watchers[key] ??= [];
    this.watchers[key]?.push(listener);
    this.get(key).then((value) => {
      listener(value);
    }).catch((e) => {
      console.error(
        `@frames.js/render: Failed to get value from storage: ${e}`
      );
    });
    return () => {
      const listeners = this.watchers[key];
      if (listeners) {
        this.watchers[key] = listeners.filter(
          (watcher) => watcher !== listener
        );
      }
    };
  };
  notifyChange = (key, value) => {
    this.watchers[key]?.forEach((listener) => {
      listener(value);
    });
  };
};

// src/identity/xmtp/use-xmtp-identity.tsx
var defaultStorage = new WebStorage();
function useXmtpIdentity({
  storage = defaultStorage,
  storageKey = "xmtpSigner"
} = {}) {
  const storageRef = (0, import_react.useRef)(storage);
  const [isLoading, setIsLoading] = (0, import_react.useState)(false);
  const [xmtpSigner, setXmtpSigner] = (0, import_react.useState)(null);
  const [xmtpClient, setXmtpClient] = (0, import_react.useState)(null);
  const config = (0, import_wagmi.useConfig)();
  const connect = (0, import_rainbowkit.useConnectModal)();
  const { address } = (0, import_wagmi.useAccount)();
  const walletSigner = (0, import_react.useMemo)(
    () => address ? {
      getAddress() {
        return Promise.resolve(address);
      },
      signMessage(message) {
        return (0, import_actions.signMessage)(config, {
          message: {
            raw: typeof message === "string" ? Buffer.from(message) : Buffer.from(message)
          }
        });
      }
    } : null,
    [address, config]
  );
  (0, import_react.useEffect)(() => {
    async function instantiateXmtpSignerAndClient() {
      const storedSigner = await storageRef.current.get(storageKey);
      if (!storedSigner) {
        return;
      }
      const signer = {
        walletAddress: storedSigner.walletAddress,
        keys: Buffer.from(storedSigner.keys, "hex")
      };
      const client = await import_xmtp_js.Client.create(null, {
        privateKeyOverride: signer.keys
      });
      setXmtpSigner(signer);
      setXmtpClient(client);
    }
    instantiateXmtpSignerAndClient().catch((e) => {
      console.error(
        "@frames.js/render: Could not instantiate the XMTP signer and client",
        e
      );
    });
  }, [storageKey]);
  const logout = (0, import_react.useCallback)(async () => {
    await storageRef.current.delete(storageKey);
    setXmtpClient(null);
    setXmtpSigner(null);
  }, [storageKey]);
  const onSignerlessFramePress = (0, import_react.useCallback)(async () => {
    try {
      setIsLoading(true);
      if (!xmtpSigner) {
        if (!walletSigner) {
          connect.openConnectModal?.();
          return;
        }
        const keys = await import_xmtp_js.Client.getKeys(walletSigner, {
          env: "dev",
          skipContactPublishing: true,
          persistConversations: false
        });
        const client = await import_xmtp_js.Client.create(null, {
          privateKeyOverride: keys
        });
        const walletAddress = (0, import_actions.getAccount)(config).address || import_viem.zeroAddress;
        await storageRef.current.set(storageKey, () => ({
          walletAddress,
          keys: Buffer.from(keys).toString("hex")
        }));
        setXmtpSigner({
          keys,
          walletAddress
        });
        setXmtpClient(client);
      }
    } catch (error) {
      console.error("@frames.js/render: API Call failed", error);
    } finally {
      setIsLoading(false);
    }
  }, [config, walletSigner, xmtpSigner, connect, storageKey]);
  const signFrameAction = (0, import_react.useCallback)(
    async (actionContext) => {
      if (!xmtpClient) {
        throw new Error("No xmtp client");
      }
      if (!address) {
        throw new Error("No address");
      }
      const framesClient = new import_frames_client.FramesClient(xmtpClient);
      const payload = await framesClient.signFrameAction({
        frameUrl: actionContext.url,
        inputText: actionContext.inputText,
        state: actionContext.state,
        buttonIndex: actionContext.buttonIndex,
        conversationTopic: actionContext.frameContext.conversationTopic,
        participantAccountAddresses: actionContext.frameContext.participantAccountAddresses,
        ...actionContext.frameContext.groupId ? { groupId: actionContext.frameContext.groupId } : {},
        ...actionContext.frameContext.groupSecret ? { groupSecret: actionContext.frameContext.groupSecret } : {},
        ...actionContext.type === "tx-data" || actionContext.type === "tx-post" ? { address: actionContext.address } : {},
        ...actionContext.type === "tx-post" ? { transactionId: actionContext.transactionId } : {}
      });
      const searchParams = new URLSearchParams({
        postType: actionContext.type === "tx-post" ? "post" : actionContext.frameButton.action,
        postUrl: actionContext.target ?? "",
        specification: "openframes"
      });
      return {
        body: payload,
        searchParams
      };
    },
    [address, xmtpClient]
  );
  return (0, import_react.useMemo)(
    () => ({
      signer: xmtpSigner,
      hasSigner: !!xmtpSigner?.keys,
      signFrameAction,
      isLoadingSigner: isLoading,
      onSignerlessFramePress,
      logout
    }),
    [isLoading, logout, onSignerlessFramePress, signFrameAction, xmtpSigner]
  );
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useXmtpIdentity
});
