"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/next/route-handlers.ts
var route_handlers_exports = {};
__export(route_handlers_exports, {
  GET: () => GET,
  POST: () => POST
});
module.exports = __toCommonJS(route_handlers_exports);

// src/next/GET.tsx
var import_parseFramesWithReports = require("frames.js/parseFramesWithReports");
var import_frames = require("frames.js");

// src/next/validators.ts
function isSpecificationValid(specification) {
  return typeof specification === "string" && ["farcaster", "openframes"].includes(specification);
}

// src/next/GET.tsx
async function GET(request) {
  try {
    const searchParams = "nextUrl" in request ? request.nextUrl.searchParams : new URL(request.url).searchParams;
    const url = searchParams.get("url");
    const specification = searchParams.get("specification") ?? "farcaster";
    const multiSpecificationEnabled = searchParams.get("multispecification") === "true";
    if (!url) {
      return Response.json({ message: "Invalid URL" }, {
        status: 400
      });
    }
    const urlRes = await fetch(url);
    const html = await urlRes.text();
    if (multiSpecificationEnabled) {
      const result2 = (0, import_parseFramesWithReports.parseFramesWithReports)({
        html,
        fallbackPostUrl: url
      });
      return Response.json(result2);
    }
    if (!isSpecificationValid(specification)) {
      return Response.json(
        { message: "Invalid specification" },
        {
          status: 400
        }
      );
    }
    const result = (0, import_frames.getFrame)({
      htmlString: html,
      url,
      specification
    });
    return Response.json(result);
  } catch (err) {
    console.error(err);
    return Response.json({ message: String(err) }, {
      status: 500
    });
  }
}

// src/next/POST.tsx
var import_frames2 = require("frames.js");
var import_parseFramesWithReports2 = require("frames.js/parseFramesWithReports");

// src/helpers.ts
async function tryCallAsync(promiseFn) {
  try {
    return promiseFn().catch((e) => {
      if (e instanceof Error) {
        return e;
      }
      return new TypeError("Unexpected error, check the console for details");
    });
  } catch (e) {
    return new TypeError("Unexpected error, check the console for details");
  }
}

// src/next/POST.tsx
function isJsonErrorObject(data) {
  return typeof data === "object" && data !== null && "message" in data && typeof data.message === "string";
}
async function POST(req) {
  try {
    const searchParams = "nextUrl" in req ? req.nextUrl.searchParams : new URL(req.url).searchParams;
    const body = await req.json();
    const isPostRedirect = searchParams.get("postType") === "post_redirect";
    const isTransactionRequest = searchParams.get("postType") === "tx";
    const postUrl = searchParams.get("postUrl");
    const multiSpecificationEnabled = searchParams.get("multispecification") === "true";
    const specification = searchParams.get("specification") ?? "farcaster";
    if (!postUrl) {
      return Response.json(
        { message: "postUrl parameter not found" },
        {
          status: 400
        }
      );
    }
    const response = await fetch(postUrl, {
      method: "POST",
      headers: {
        Accept: "application/json",
        "Content-Type": "application/json"
      },
      redirect: isPostRedirect ? "manual" : void 0,
      body: JSON.stringify(body)
    });
    if (response.status >= 500) {
      const jsonError = await tryCallAsync(
        () => response.clone().json()
      );
      if (jsonError instanceof Error) {
        return Response.json(
          { message: jsonError.message },
          { status: response.status }
        );
      }
      if (isJsonErrorObject(jsonError)) {
        return Response.json(
          { message: jsonError.message },
          { status: response.status }
        );
      }
      console.error(jsonError);
      return Response.json(
        {
          message: `Frame server returned an unexpected error.`
        },
        { status: 500 }
      );
    }
    if (response.status === 302) {
      const location = response.headers.get("location");
      if (!location) {
        return Response.json(
          {
            message: "Frame server returned a redirect without a location header"
          },
          { status: 500 }
        );
      }
      return Response.json(
        {
          location
        },
        { status: 302 }
      );
    } else if (isPostRedirect) {
      return Response.json(
        {
          message: "Frame server did not return a 302 redirect"
        },
        { status: 500 }
      );
    }
    if (response.status >= 400 && response.status < 500) {
      const jsonError = await tryCallAsync(
        () => response.clone().json()
      );
      if (jsonError instanceof Error) {
        return Response.json(
          { message: jsonError.message },
          { status: response.status }
        );
      }
      if (isJsonErrorObject(jsonError)) {
        return Response.json(
          { message: jsonError.message },
          { status: response.status }
        );
      }
      console.error(jsonError);
      return Response.json(
        {
          message: `Frame server returned an unexpected error.`
        },
        { status: response.status }
      );
    }
    if (response.status !== 200) {
      return Response.json(
        {
          message: `Frame server returned a non-200 status code: ${response.status}`
        },
        { status: 500 }
      );
    }
    if (isTransactionRequest) {
      const transaction = await tryCallAsync(
        () => response.clone().json()
      );
      if (transaction instanceof Error) {
        return Response.json(
          { message: transaction.message },
          { status: 500 }
        );
      }
      return Response.json(transaction);
    }
    const html = await response.text();
    if (multiSpecificationEnabled) {
      const result2 = (0, import_parseFramesWithReports2.parseFramesWithReports)({
        html,
        fallbackPostUrl: body.untrustedData.url,
        fromRequestMethod: "POST"
      });
      return Response.json(result2);
    }
    if (!isSpecificationValid(specification)) {
      return Response.json(
        {
          message: "Invalid specification"
        },
        { status: 400 }
      );
    }
    const result = (0, import_frames2.getFrame)({
      htmlString: html,
      url: body.untrustedData.url,
      fromRequestMethod: "POST",
      specification
    });
    return Response.json(result);
  } catch (err) {
    console.error(err);
    return Response.json({ message: String(err) }, {
      status: 500
    });
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  GET,
  POST
});
