"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/ui/frame.base.tsx
var frame_base_exports = {};
__export(frame_base_exports, {
  BaseFrameUI: () => BaseFrameUI
});
module.exports = __toCommonJS(frame_base_exports);
var import_react = require("react");

// src/ui/utils.ts
function isPartialFrameStackItem(stackItem) {
  return stackItem.status === "done" && stackItem.frameResult.status === "failure" && !!stackItem.frameResult.frame.image && !!stackItem.frameResult.frame.buttons && stackItem.frameResult.frame.buttons.length > 0;
}
function getErrorMessageFromFramesStackItem(item) {
  if (item.status === "message") {
    return item.message;
  }
  if (item.requestError instanceof Error) {
    return item.requestError.message;
  }
  return "An error occurred";
}

// src/ui/frame.base.tsx
function defaultMessageHandler() {
}
function BaseFrameUI({
  frameState,
  components,
  theme,
  allowPartialFrame = false,
  enableImageDebugging = false,
  // eslint-disable-next-line no-console -- provide at least some feedback to the user
  onError = console.error,
  onMessage = defaultMessageHandler,
  createElement = import_react.createElement
}) {
  const [isImageLoading, setIsImageLoading] = (0, import_react.useState)(true);
  const { currentFrameStackItem } = frameState;
  const rootRef = (0, import_react.useRef)(null);
  const rootDimensionsRef = (0, import_react.useRef)();
  const onImageLoadEnd = (0, import_react.useCallback)(() => {
    setIsImageLoading(false);
  }, []);
  (0, import_react.useEffect)(() => {
    if (currentFrameStackItem?.status === "pending") {
      setIsImageLoading(true);
    }
  }, [currentFrameStackItem?.status]);
  const onErrorRef = (0, import_react.useRef)(onError);
  onErrorRef.current = onError;
  (0, import_react.useEffect)(() => {
    if (currentFrameStackItem?.status === "requestError") {
      onErrorRef.current(currentFrameStackItem.requestError);
    }
  }, [currentFrameStackItem]);
  const onMessageRef = (0, import_react.useRef)(onMessage);
  onMessageRef.current = onMessage;
  (0, import_react.useEffect)(() => {
    if (currentFrameStackItem?.status === "message") {
      onMessageRef.current({
        message: currentFrameStackItem.message,
        status: currentFrameStackItem.type === "info" ? "message" : "error"
      });
    }
  }, [currentFrameStackItem]);
  if (!frameState.homeframeUrl) {
    return components.Error(
      { message: "Missing frame url" },
      theme?.Error || {}
    );
  }
  if (!currentFrameStackItem) {
    return null;
  }
  let frameUiState;
  const previousFrameStackItem = frameState.framesStack[frameState.framesStack.length - 1];
  const previousFrame = previousFrameStackItem?.status === "done" ? previousFrameStackItem.frameResult.frame : null;
  switch (currentFrameStackItem.status) {
    case "requestError": {
      if ("sourceFrame" in currentFrameStackItem.request && currentFrameStackItem.request.sourceFrame) {
        frameUiState = {
          status: "complete",
          frame: currentFrameStackItem.request.sourceFrame,
          isImageLoading,
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      } else {
        return components.Error(
          {
            message: "Failed to load frame",
            error: currentFrameStackItem.requestError
          },
          theme?.Error || {}
        );
      }
      break;
    }
    case "message":
      if (!currentFrameStackItem.request.sourceFrame) {
        return components.Message(
          {
            message: currentFrameStackItem.message,
            status: currentFrameStackItem.type === "error" ? "error" : "message"
          },
          theme?.Message ?? {}
        );
      }
      frameUiState = {
        status: "complete",
        frame: currentFrameStackItem.request.sourceFrame,
        isImageLoading,
        id: currentFrameStackItem.timestamp.getTime(),
        frameState
      };
      break;
    case "doneRedirect": {
      if (!currentFrameStackItem.request.sourceFrame) {
        frameUiState = {
          status: "loading",
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      } else {
        frameUiState = {
          status: "complete",
          frame: currentFrameStackItem.request.sourceFrame,
          isImageLoading,
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      }
      break;
    }
    case "done": {
      if (currentFrameStackItem.frameResult.status === "success") {
        frameUiState = {
          status: "complete",
          frame: currentFrameStackItem.frameResult.frame,
          debugImage: enableImageDebugging ? currentFrameStackItem.frameResult.framesDebugInfo?.image : void 0,
          isImageLoading,
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      } else if (isPartialFrameStackItem(currentFrameStackItem) && allowPartialFrame) {
        frameUiState = {
          status: "partial",
          frame: currentFrameStackItem.frameResult.frame,
          debugImage: enableImageDebugging ? currentFrameStackItem.frameResult.framesDebugInfo?.image : void 0,
          isImageLoading,
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      } else {
        return components.Error(
          { message: "Invalid frame" },
          theme?.Error || {}
        );
      }
      break;
    }
    case "pending": {
      frameUiState = {
        status: "loading",
        id: currentFrameStackItem.timestamp.getTime(),
        frameState
      };
      break;
    }
  }
  const isLoading = frameUiState.status === "loading" || frameUiState.isImageLoading;
  const buttonsProps = frameUiState.status === "loading" || !frameUiState.frame.buttons || frameUiState.frame.buttons.length === 0 ? null : frameUiState.frame.buttons.map((frameButton, index) => ({
    frameState: frameUiState,
    frameButton,
    index,
    isDisabled: false,
    onPress() {
      rootDimensionsRef.current = rootRef.current?.computeDimensions();
      Promise.resolve(
        frameState.onButtonPress(
          // @todo change the type onButtonPress to accept partial frame as well because that can happen if partial frames are enabled
          frameUiState.frame,
          frameButton,
          index
        )
      ).catch((error) => {
        console.error(error);
      });
    }
  }));
  return components.Root(
    {
      createElement,
      frameState: frameUiState,
      dimensions: isLoading ? rootDimensionsRef.current ?? null : null,
      ref: rootRef,
      loadingScreen: isLoading ? components.LoadingScreen(
        {
          frameState: frameUiState,
          dimensions: rootDimensionsRef.current ?? null
        },
        theme?.LoadingScreen || {}
      ) : null,
      buttonsContainer: buttonsProps ? components.ButtonsContainer(
        {
          frameState: frameUiState,
          buttons: buttonsProps.map(
            (buttonProps) => components.Button(
              buttonProps,
              theme?.Button || {}
            )
          ),
          buttonsProps
        },
        theme?.ButtonsContainer || {}
      ) : null,
      imageContainer: components.ImageContainer(
        {
          frameState: frameUiState,
          aspectRatio: (frameUiState.status === "loading" ? previousFrame?.imageAspectRatio : frameUiState.frame.imageAspectRatio) ?? "1.91:1",
          image: components.Image(
            frameUiState.status === "loading" ? {
              status: "frame-loading",
              frameState: frameUiState,
              onImageLoadEnd,
              aspectRatio: previousFrame?.imageAspectRatio ?? "1.91:1"
            } : {
              status: "frame-loading-complete",
              frameState: frameUiState,
              src: frameUiState.debugImage ?? frameUiState.frame.image,
              aspectRatio: frameUiState.frame.imageAspectRatio ?? "1.91:1",
              onImageLoadEnd
            },
            theme?.Image || {}
          ),
          messageTooltip: currentFrameStackItem.status === "message" ? components.MessageTooltip(
            {
              frameState: frameUiState,
              status: currentFrameStackItem.type === "error" ? "error" : "message",
              message: getErrorMessageFromFramesStackItem(
                currentFrameStackItem
              )
            },
            theme?.MessageTooltip || {}
          ) : null
        },
        theme?.ImageContainer || {}
      ),
      textInputContainer: frameUiState.status === "loading" || typeof frameUiState.frame.inputText !== "string" ? null : components.TextInputContainer(
        {
          frame: frameUiState,
          textInput: components.TextInput(
            {
              // @TODO provide previous frame to pending state so we can remove loading check and render skeletons?
              isDisabled: false,
              frameState: frameUiState,
              placeholder: frameUiState.frame.inputText,
              onChange(text) {
                frameState.setInputText(text);
              },
              value: frameState.inputText
            },
            theme?.TextInput || {}
          )
        },
        theme?.TextInputContainer || {}
      )
    },
    theme?.Root || {}
  );
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  BaseFrameUI
});
