import {
  useFetchFrame
} from "./chunk-JRMX7FFK.js";
import {
  useFrameState
} from "./chunk-5XB2T5J3.js";
import {
  useFreshRef
} from "./chunk-KTZM67PO.js";
import "./chunk-6AQ6N3W7.js";
import "./chunk-7RL4LVAH.js";
import "./chunk-6C3VEZWH.js";

// src/unstable-use-frame.tsx
import { useCallback, useEffect, useMemo, useState } from "react";
function onMintFallback({ target }) {
  console.log("Please provide your own onMint function to useFrame() hook.");
  const message = `Mint requested: ${target}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
}
function onConnectWalletFallback() {
  throw new Error(
    "Please implement this function in order to use transactions"
  );
}
async function onTransactionFallback({
  transactionData
}) {
  console.log(
    "Please provide your own onTransaction function to useFrame() hook."
  );
  const message = `Requesting a transaction on chain with ID ${transactionData.chainId} with the following params: ${JSON.stringify(
    transactionData.params,
    null,
    2
  )}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
  return null;
}
async function onSignatureFallback({
  signatureData
}) {
  console.log(
    "Please provide your own onSignature function to useFrame() hook."
  );
  const message = `Requesting a signature on chain with ID ${signatureData.chainId} with the following params: ${JSON.stringify(
    signatureData.params,
    null,
    2
  )}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
  return null;
}
function handleRedirectFallback(location) {
  console.log(
    "Please provide your own onRedirect function to useFetchFrame() hook."
  );
  const message = `You are about to be redirected to ${location.toString()}`;
  if (typeof window !== "undefined") {
    if (window.confirm(message)) {
      window.open(location, "_blank")?.focus();
    }
  } else {
    console.log(message);
  }
}
function handleLinkButtonClickFallback(button) {
  console.log(
    "Please provide your own onLinkButtonClick function to useFrame() hook."
  );
  if (typeof window !== "undefined") {
    if (window.confirm(`You are about to be redirected to ${button.target}`)) {
      parent.window.open(button.target, "_blank");
    }
  } else {
    console.log(`Link button with target ${button.target} clicked.`);
  }
}
function defaultComposerFormActionHandler() {
  throw new Error('Please implement your own "onComposerFormAction" handler');
}
function validateLinkButtonTarget(target) {
  const locationUrl = new URL(target);
  if (locationUrl.protocol !== "http:" && locationUrl.protocol !== "https:") {
    throw new Error(
      `Redirect location ${locationUrl.toString()} is not a valid HTTP or HTTPS URL.`
    );
  }
  return true;
}
function useFrame({
  homeframeUrl,
  onMint = onMintFallback,
  onTransaction = onTransactionFallback,
  transactionDataSuffix,
  onConnectWallet = onConnectWalletFallback,
  onSignature = onSignatureFallback,
  connectedAddress,
  frame,
  /** Ex: /frames */
  frameActionProxy,
  /** Ex: /frames */
  frameGetProxy,
  extraButtonRequestPayload,
  resolveSpecification,
  resolveCastOrComposerActionSigner,
  onError,
  onLinkButtonClick = handleLinkButtonClickFallback,
  onRedirect = handleRedirectFallback,
  fetchFn = (...args) => fetch(...args),
  onComposerFormAction = defaultComposerFormActionHandler,
  onTransactionDataError,
  onTransactionDataStart,
  onTransactionDataSuccess,
  onTransactionError,
  onTransactionProcessingError,
  onTransactionProcessingStart,
  onTransactionProcessingSuccess,
  onTransactionStart,
  onTransactionSuccess
}) {
  const resolveCastOrComposerActionSignerRef = useFreshRef(
    resolveCastOrComposerActionSigner
  );
  const [inputText, setInputText] = useState("");
  const inputTextRef = useFreshRef(inputText);
  const [frameState, frameStateAPI] = useFrameState({
    resolveSpecification,
    initialFrameUrl: homeframeUrl,
    initialParseResult: frame
  });
  const frameStateRef = useFreshRef(frameState);
  const {
    clear: clearFrameState,
    dispatch: dispatchFrameState,
    reset: resetFrameState
  } = frameStateAPI;
  const fetchFrame = useFetchFrame({
    frameState,
    frameStateAPI,
    frameActionProxy,
    frameGetProxy,
    onTransaction,
    transactionDataSuffix,
    onSignature,
    extraButtonRequestPayload,
    onError,
    fetchFn,
    onRedirect,
    onComposerFormAction,
    onTransactionDataError,
    onTransactionDataStart,
    onTransactionDataSuccess,
    onTransactionError,
    onTransactionProcessingError,
    onTransactionProcessingStart,
    onTransactionProcessingSuccess,
    onTransactionStart,
    onTransactionSuccess
  });
  const fetchFrameRef = useFreshRef(fetchFrame);
  const onErrorRef = useFreshRef(onError);
  useEffect(() => {
    if (!homeframeUrl) {
      clearFrameState();
      return;
    }
    if (!frame) {
      fetchFrameRef.current(
        {
          url: homeframeUrl,
          method: "GET"
        },
        // tell the fetchFrame function to clear the stack because this is called only on initial render
        // and there could potentially be a pending object returned from SSR
        true
      ).catch((e) => {
        console.error(e);
      });
    } else {
      resetFrameState({
        homeframeUrl,
        parseResult: frame
      });
    }
  }, [frame, homeframeUrl, clearFrameState, fetchFrameRef, resetFrameState]);
  const onPostButton = useCallback(
    async function onPostButton2({
      currentFrame,
      buttonIndex,
      postInputText,
      frameButton,
      target,
      state,
      fetchFrameOverride
    }) {
      const currentState = frameStateRef.current;
      if (currentState.type === "not-initialized") {
        const error = new Error(
          "Cannot perform post/post_redirect without a frame"
        );
        console.error(`@frames.js/render: ${error.message}`);
        onErrorRef.current?.(error);
        return;
      }
      if (!currentState.signerState.hasSigner) {
        const error = new Error("Missing signer");
        console.error(`@frames.js/render: ${error.message}`);
        onErrorRef.current?.(error);
        return;
      }
      const _fetchFrame = fetchFrameOverride ?? fetchFrameRef.current;
      await _fetchFrame({
        frameButton,
        isDangerousSkipSigning: false,
        method: "POST",
        signerStateActionContext: {
          inputText: postInputText,
          signer: currentState.signerState.signer,
          frameContext: currentState.frameContext,
          url: currentState.homeframeUrl,
          target,
          frameButton,
          buttonIndex,
          state
        },
        sourceFrame: currentFrame
      });
    },
    [fetchFrameRef, frameStateRef, onErrorRef]
  );
  const onConnectWalletRef = useFreshRef(onConnectWallet);
  const connectedAddressRef = useFreshRef(connectedAddress);
  const onTransactionButton = useCallback(
    async function onTransactionButton2({
      currentFrame,
      buttonIndex,
      postInputText,
      frameButton
    }) {
      const state = frameStateRef.current;
      if (state.type === "not-initialized") {
        const error = new Error("Cannot perform transaction without a frame");
        console.error(`@frames.js/render: ${error.message}`);
        onErrorRef.current?.(error);
        return;
      }
      if (!state.signerState.hasSigner) {
        const error = new Error("Missing signer");
        console.error(`@frames.js/render: ${error.message}`);
        onErrorRef.current?.(error);
        return;
      }
      if (!connectedAddressRef.current) {
        try {
          onConnectWalletRef.current();
        } catch (e) {
          onErrorRef.current?.(e instanceof Error ? e : new Error(String(e)));
          console.error(`@frames.js/render: ${String(e)}`);
        }
        return;
      }
      await fetchFrameRef.current({
        frameButton,
        isDangerousSkipSigning: false,
        method: "POST",
        signerStateActionContext: {
          type: "tx-data",
          inputText: postInputText,
          signer: state.signerState.signer,
          frameContext: state.frameContext,
          address: connectedAddressRef.current,
          url: state.homeframeUrl,
          target: frameButton.target,
          frameButton,
          buttonIndex,
          state: currentFrame.state
        },
        sourceFrame: currentFrame
      });
    },
    [
      frameStateRef,
      connectedAddressRef,
      fetchFrameRef,
      onErrorRef,
      onConnectWalletRef
    ]
  );
  const onButtonPress = useCallback(
    async function onButtonPress2(currentFrame, frameButton, index, fetchFrameOverride) {
      switch (frameButton.action) {
        case "link": {
          try {
            validateLinkButtonTarget(frameButton.target);
          } catch (error) {
            if (error instanceof Error) {
              onErrorRef.current?.(error);
            }
            return;
          }
          onLinkButtonClick(frameButton);
          break;
        }
        case "mint": {
          onMint({
            frameButton,
            target: frameButton.target,
            frame: currentFrame
          });
          break;
        }
        case "tx": {
          await onTransactionButton({
            frameButton,
            buttonIndex: index + 1,
            postInputText: currentFrame.inputText !== void 0 ? inputTextRef.current : void 0,
            currentFrame
          });
          break;
        }
        case "post":
        case "post_redirect": {
          try {
            const target = frameButton.target || frameButton.post_url || currentFrame.postUrl || homeframeUrl;
            if (!target) {
              onErrorRef.current?.(new Error(`Missing target`));
              return;
            }
            try {
              validateLinkButtonTarget(target);
            } catch (error) {
              if (error instanceof Error) {
                onErrorRef.current?.(error);
              }
              return;
            }
            await onPostButton({
              currentFrame,
              frameButton,
              /** https://docs.farcaster.xyz/reference/frames/spec#handling-clicks
              
                      POST the packet to fc:frame:button:$idx:action:target if present
                      POST the packet to fc:frame:post_url if target was not present.
                      POST the packet to or the frame's embed URL if neither target nor action were present.
                      */
              target,
              buttonIndex: index + 1,
              postInputText: currentFrame.inputText !== void 0 ? inputTextRef.current : void 0,
              state: currentFrame.state,
              fetchFrameOverride
            });
            setInputText("");
          } catch (err) {
            if (err instanceof Error) {
              onErrorRef.current?.(err);
            }
            console.error(err);
          }
          break;
        }
        default:
          throw new Error("Unrecognized frame button action");
      }
    },
    [
      homeframeUrl,
      inputTextRef,
      onErrorRef,
      onLinkButtonClick,
      onMint,
      onPostButton,
      onTransactionButton
    ]
  );
  const onCastActionButtonPress = useCallback(
    async function onActionButtonPress(arg) {
      const { signerState: signerState2, frameContext } = resolveCastOrComposerActionSignerRef.current({
        action: {
          type: "cast",
          action: arg.castAction
        }
      });
      if (!signerState2.hasSigner) {
        await signerState2.onSignerlessFramePress();
        return;
      }
      return fetchFrame(
        {
          method: "CAST_ACTION",
          signerState: signerState2,
          action: arg.castAction,
          isDangerousSkipSigning: false,
          signerStateActionContext: {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment -- we trust the signerState
            signer: signerState2.signer,
            frameContext,
            url: arg.castAction.url,
            target: arg.castAction.url,
            buttonIndex: 1
          }
        },
        arg.clearStack
      );
    },
    [fetchFrame, resolveCastOrComposerActionSignerRef]
  );
  const onComposerActionButtonPress = useCallback(
    async function onActionButtonPress(arg) {
      const { signerState: signerState2, frameContext } = resolveCastOrComposerActionSignerRef.current({
        action: {
          type: "compose",
          action: arg.castAction,
          composerActionState: arg.composerActionState
        }
      });
      if (!signerState2.hasSigner) {
        await signerState2.onSignerlessFramePress();
        return;
      }
      return fetchFrame(
        {
          method: "COMPOSER_ACTION",
          signerState: signerState2,
          action: arg.castAction,
          isDangerousSkipSigning: false,
          composerActionState: arg.composerActionState,
          signerStateActionContext: {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment -- we trust the signerState
            signer: signerState2.signer,
            frameContext,
            url: arg.castAction.url,
            target: arg.castAction.url,
            buttonIndex: 1
          }
        },
        arg.clearStack
      );
    },
    [fetchFrame, resolveCastOrComposerActionSignerRef]
  );
  const { stack } = frameState;
  const { signerState, specification } = frameState.type === "initialized" ? frameState : { signerState: void 0, specification: void 0 };
  return useMemo(() => {
    return {
      signerState,
      specification,
      inputText,
      setInputText,
      clearFrameStack: clearFrameState,
      dispatchFrameStack: dispatchFrameState,
      reset: resetFrameState,
      onButtonPress,
      fetchFrame,
      homeframeUrl,
      framesStack: stack,
      currentFrameStackItem: stack[0],
      onCastActionButtonPress,
      onComposerActionButtonPress
    };
  }, [
    signerState,
    specification,
    inputText,
    clearFrameState,
    dispatchFrameState,
    onButtonPress,
    resetFrameState,
    fetchFrame,
    homeframeUrl,
    stack,
    onCastActionButtonPress,
    onComposerActionButtonPress
  ]);
}
export {
  useFrame
};
